/*
 * Created on 2004/08/07
 *
 */
package jnet.hirommy.core;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.*;

import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.util.InfoLogger;
import jnet.hirommy.util.ViewerHandler;
import jnet.hirommy.util.ViewerHandlerImpl;

/**
 * This class for receiving process.
 * If information is received, it will record on a log and GpsViewer will be updated.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class InfoReceiver implements Runnable, NetworkConst {
	/**
	 * true:start receive, false:stop receive
	 */
	private boolean receivigProcess = false;
	
	/**
	 * This is a method for starting a receiving process.
	 * If information is received, it will record on a log and GpsViewer will be updated.
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			// UDP socket for receiving
			DatagramSocket sock = new DatagramSocket(PORT);	
			
			// UDP packet for receiving
			DatagramPacket packet =
					new DatagramPacket(new byte[MAX_PACKET_SIZE], MAX_PACKET_SIZE);
			
			// start receiving process.
			receivigProcess = true;		// true:start receive, false:stop receive
			while (receivigProcess) {
				sock.receive(packet);	// receiving packet from other terminal.
				
				// When it is except the packet which this terminal broadcast.
				// The packet which this terminal transmitted does not receive.
				if (!SystemProperty.getInstance().getAddr().equals(packet.getAddress().getHostAddress())) {
					// decode infomation from received Hirommy Packet.
					// creation of byte array input stream from byte array that this terminal received packet.
					ByteArrayInputStream byteArrayInputStream
							= new ByteArrayInputStream(packet.getData(),
														packet.getOffset(),
														packet.getLength());
					// creation of object input stream from byte array input stream.
					ObjectInputStream objectInputStream = new ObjectInputStream(byteArrayInputStream);
					// deSerialization of HirommyPacket from ObjectInputStream.
					HirommyPacket hirommyPacket = (HirommyPacket)objectInputStream.readObject();
					
					// It processes, when the IP address set as the Hirommy packet differs from its address.
					// The packet which itself who has been responsed by the echo server transmitted is eliminated.
					if (!SystemProperty.getInstance().getAddr().equals(hirommyPacket.getIpAddr())) {
						
						// save infomation to InfoLogger and update GpsViewer.
						InfoLogger infoLogger = InfoLogger.getInstance();		// get InfoLogger instance.
						infoLogger.logReceivedHirommyPacket(hirommyPacket);		// save infomation.
						
						// update Viewer.
						ViewerHandler viewerHandler = new ViewerHandlerImpl();
						viewerHandler.repaintReceivingInfoViewer(hirommyPacket);	// update ReceivingInfoViewInternalFrame
						
						// if reflect mode is true and it is in a RIFUREKUTO radius, 
						// its information will be transmitted when information is received from other terminals.
						if (SystemProperty.getInstance().isReflect() 
								&& SystemProperty.getInstance().isInReflectRadius(hirommyPacket)) {
							// send latest my infomation.
							InfoSender.sendPacket((HirommyPacket)InfoLogger.getInstance().getMyInfo().getLast());
						}
						
						// show received infomation.
						System.out.println("Receiving : "+ hirommyPacket.toString());
					}
						
					objectInputStream.close();							// close object input stream.
					byteArrayInputStream.close();						// close byte array stream.
					
					packet.setLength(MAX_PACKET_SIZE);					// reset packet size
				}
			}
			
			sock.close();		// close socket.
			
		} catch (SocketException e) {
			System.out.println("Error : An error occurs in UDP.");
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("Error : An error occurs in IO.");
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			System.out.println("Error : An error occurs in Serialization");
			e.printStackTrace();
		} catch (NullPointerException e) {
			// no process.
		}
	}
	
	/**
	 * The reception end which is information from other terminals.
	 *
	 */
	public void start() {
		new Thread(this, "ReceivingProcess").start();
	}
	
	/**
	 * A receiving process is ended by this method.
	 *
	 */
	public void close() {
		receivigProcess = false;
		System.out.println("close UDP port for receiving Hirommy Packet.");
	}
}
