/*
 * Created on 2004/08/07
 *
 */
package jnet.hirommy.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.*;

import jnet.hirommy.info.HirommyPacket;

/**
 * This class send HirommyPacket.
 * UDP is used for transmission of a packet in Hirommy.
 * Since UDP is unreliable, 
 * it transmits the same information twice and raises reliability.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class InfoSender implements NetworkConst {
	/**
	 * between send GPS infomatin (ms)
	 */
	private static final int SLEEP_TIME = 500;
	
	/**
	 * This method for sending HirommyPacket.
	 * 
	 * @param hirommyPacket HirommyPacket
	 */
	public static void sendPacket(HirommyPacket hirommyPacket) {
		try {
			// get destintion IP Address.
			// Hirommy broadcast a packet, so A broadcast address is set as the target address.
			InetAddress destAddr = InetAddress.getByName("255.255.255.255");
			
			// Creation of a byte array stream.
			ByteArrayOutputStream byteArrayOutuptStream = new ByteArrayOutputStream();
			// Creation of a object output stream.
			ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutuptStream);
			// Serializable HirommyPacket Object.
			objectOutputStream.writeObject(hirommyPacket);
			// create byteData to send UDP Packet from byte array stream that HirommyPacket is serialized.
			byte[] bytesToSend = byteArrayOutuptStream.toByteArray();
			
			// creation of a DatagramPacket to send HirommyPacket.
			DatagramPacket packet = new DatagramPacket(bytesToSend, bytesToSend.length, destAddr, PORT);
			// UDP socket for sending GPS infomation
			DatagramSocket sock = new DatagramSocket();	
			
			// send Hirommy Packet with UDPsocket.
			sock.send(packet);				// send latest my Hirommy Packet.
			Thread.sleep(SLEEP_TIME);		// sleep.
			sock.send(packet);				// send same packet.
			
		} catch (UnknownHostException e) {
			System.out.println("Error : Acquisition of a destination address goes wrong.");
			e.printStackTrace();
		} catch (SocketException e) {
			System.out.println("Error : An error occurs in UDP.");
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("Error : An error occurs IOException.");
			e.printStackTrace();
		} catch (InterruptedException e) {
			System.out.println("Error : An error occurs InterruptedException.");
			e.printStackTrace();
		}
		
		
	}
}
