/*
 * Created on 2004/08/07
 *
 */
package jnet.hirommy.core;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.LinkedList;

import jnet.hirommy.info.HirommyPacket;
import jnet.its.ivc.predict.PredictionAlgorithm;
import jnet.its.ivc.predict.SimplePredictionMethod;
import jnet.its.ivc.reaction.ReactionAlgorithm;
import jnet.its.ivc.reaction.SimpleReactionMethod;
import jnet.its.vpec.reflect.ReflectAlgorithm;
import jnet.its.vpec.reflect.SimpleReflectMethod;

/**
 * This class expresses My Property.
 * As for this class, only one exists in this program.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class SystemProperty {
	
	private static SystemProperty myProperty;
	
	/**
	 * IPAddress
	 */
	private String addr;
	
	/**
	 * Host Name
	 */
	private String name;
	
	/**
	 * true:reflect, false:don't reflect
	 */
	private boolean reflect = false;
	
	/**
	 * reflect radius
	 */
	private int reflectRadius = 10;
	
	/**
	 * Serial Port number for GPS receiver
	 */
	private String serialPortNumber = "COM1";
	
	// TODO setting reflect method.
	/**
	 * Algorithm for reflect.
	 */
	private ReflectAlgorithm reflectAlgorithm =
			new SimpleReflectMethod();
	
	// TODO setting prediction method.
	/**
	 * Algorithm for prediction.
	 */
	private PredictionAlgorithm predictionAlgorithm =
			new SimplePredictionMethod();
	
	// TODO setting reaction method and radius.
	/**
	 * reaction radius
	 */
	private double reactionRadius = 10;
	
	/**
	 * Algorithm for reaction method.
	 */
	private ReactionAlgorithm reactionAlgorithsm = 
			new SimpleReactionMethod(reactionRadius);
	
	/**
	 * This is default constractor.
	 * 
	 * @throws Exception
	 */
	private SystemProperty() {
		try {
			// get LocalHost infomation.
			InetAddress iAdder = InetAddress.getLocalHost();
			// set my IP Address.
			this.addr = iAdder.getHostAddress();
			// set my Hostname.
			name = iAdder.getHostName();
		} catch (UnknownHostException e) {
			System.out.println("Error : Acquisition of a LocalHost information goes wrong.");
			e.printStackTrace();
		}
	}
	
	/**
	 * The method which investigates whether a target terminal is in a reflect radius.
	 * 
	 * @param hp Hirommy Packet.
	 * @return true : target is in reflect radius.
	 */
	public boolean isInReflectRadius(HirommyPacket hp) {
		return reflectAlgorithm.isInReflectRadius(hp);
	}
	
	/**
	 * This method calculates the prediction value of a terminal. 
	 * the prediction value is contained in order of A
	 * after [ of a head ] 1 second, 2 seconds....
	 * 
	 * @param lists The past position information list.
	 * @return predict position of terminal.
	 */
	public LinkedList predictPositin(LinkedList lists) {
		return predictionAlgorithm.predict(lists);
	}
	
	/**
	 * This method returns whether a partner terminal calculates 
	 * whether it goes into the reaction domain.
	 * 
	 * @param myHirommyPacket
	 * @param targetHirommyPacket
	 * @return react or not.
	 */
	public boolean isReaction(HirommyPacket myHirommyPacket, HirommyPacket targetHirommyPacket) {
		return reactionAlgorithsm.isReact(myHirommyPacket, targetHirommyPacket);
	}
	
	/**
	 * This method is for getting myTerminalInfo instance.
	 * @return MyProperty instance.
	 * @throws Exception
	 */
	public static synchronized SystemProperty getInstance() {
		if(myProperty == null) {
			myProperty = new SystemProperty();
		}
		return myProperty;
	}
	
	/**
	 * This method returns that this system use Reflect or not.
	 * 
	 * @return Returns the reflect.
	 */
	public boolean isReflect() {
		return reflect;
	}
	
	/**
	 * This method is for setting Reflect mode on or off.
	 * 
	 * @param reflect The reflect to set.
	 */
	public void setReflect(boolean reflect) {
		this.reflect = reflect;
	}
	
	/**
	 * This method returns serial port number for getting GPS infomation from satelaite.
	 * 
	 * @return Returns the serialPortNumber.
	 */
	public String getSerialPortNumber() {
		return serialPortNumber;
	}
	/**
	 * This method is for setting serial port number for getting GPS infomation from satelaite.
	 * 
	 * @param serialPortNumber The serialPortNumber to set.
	 */
	
	public void setSerialPortNumber(String serialPortNumber) {
		this.serialPortNumber = serialPortNumber;
	}
	/**
	 * This method returns IP address of this system.
	 * 
	 * @return Returns the addr.
	 */
	public String getAddr() {
		return addr;
	}
	
	/**
	 * This method returns Host name of this system.
	 * 
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * This method retuerns reflectRadius.
	 * 
	 * @return Returns the reflectRadius.
	 */
	public int getReflectRadius() {
		return reflectRadius;
	}
	
	/**
	 * This is for setting reflect radius.
	 * 
	 * @param reflectRadius The reflectRadius to set.
	 */
	public void setReflectRadius(int reflectRadius) {
		this.reflectRadius = reflectRadius;
	}
	/**
	 * This method retuerns reaction Radius.
	 * 
	 * @return Returns the reactionRadius.
	 */
	public double getReactionRadius() {
		return reactionRadius;
	}
	/**
	 * This is for setting reflect reaction radius.
	 * 
	 * @param reactionRadius The reactionRadius to set.
	 */
	public void setReactionRadius(double reactionRadius) {
		this.reactionRadius = reactionRadius;
	}
	/**
	 * This is for setting prediction algorithm.
	 * 
	 * @param predictionAlgorithm The predictionAlgorithm to set.
	 */
	public void setPredictionAlgorithm(PredictionAlgorithm predictionAlgorithm) {
		this.predictionAlgorithm = predictionAlgorithm;
	}
	/**
	 * This is for setting reaction algorithm.
	 * 
	 * @param reactionAlgorithsm The reactionAlgorithsm to set.
	 */
	public void setReactionAlgorithsm(ReactionAlgorithm reactionAlgorithsm) {
		this.reactionAlgorithsm = reactionAlgorithsm;
	}
	/**
	 * This is for setting reflect algorithm
	 * 
	 * @param reflectAlgorithm The reflectAlgorithm to set.
	 */
	public void setReflectAlgorithm(ReflectAlgorithm reflectAlgorithm) {
		this.reflectAlgorithm = reflectAlgorithm;
	}
}
