/*
 * Created on 2004/08/10
 *
 */
package jnet.hirommy.gui;

import java.awt.*;
import java.awt.geom.Line2D;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;

import javax.swing.*;

import jnet.hirommy.util.TargetTerminal;
import jnet.hirommy.core.SystemProperty;
import jnet.hirommy.info.Building;
import jnet.hirommy.info.BuildingHandler;
import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.info.MyProfile;
import jnet.hirommy.util.DistanceCalculater;
import jnet.hirommy.util.InfoLogger;
import jnet.hirommy.util.Theme;
import jnet.hirommy.util.ThemeImpl;
import jnet.its.ivc.predict.PredictPosition;

/**
 * This class dsplays relative of position between your and other.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class GpsViewer extends JInternalFrame implements Theme {
	
	/**
	 * size of window
	 */
	public  static final double WINDOW_SIZE = 650;
	
	/**
	 * top inset
	 */
	public  static final double TOP_INSET = 25;
	
	/**
	 * This is live of terminal (ms)
	 */
	private static final int TERMINAL_LIVE = 900000;
	
	/**
	 * This is sise of circle radias.
	 */
	private double radiusSize = 100;
	
	/**
	 * This is size of target terinal for display this window.
	 */
	private int targetSize = 8;
	
	/**
	 * This is flag which dysplay target description or not.
	 */
	private boolean targetDescription = true;
	
	/**
	 * This is img valiable.
	 */
	private Image img;
	
	/**
	 * This is file name of background imagge of GPS Viewer
	 */
	private String backgroundImg;
	
	/**
	 * This is flag that draw image or not.
	 * ture:use image, false:don't use image
	 */
	private boolean backgroundImgFlag = false;
	
	/**
	 * This is flag that draw building or not.
	 * true:show building, false:don't show building
	 */
	private boolean buldingFlag = true;
	
	/**
	 * This is flag that draw direction or not.
	 * true:show direction, false;don't show direction
	 */
	private boolean directionFlag = true;	
	
	/**
	 * This is flag that draw prediction position or not.
	 * ture:show precition position, false:don't show
	 */
	private boolean predictionFlag = true;
	
	/**
	 * This is a magnitude of my direction.
	 */
	private final int MAGNI = 5;
	
	/**
	 * This is that show react domain or not.
	 */
	private boolean reactRadius = false;
	
	/**
	 * This is color object that display prediction position.
	 */
	private Color predictedColor1 = new Color(204, 204, 204);
	
	/**
	 * This is a default constractor.
	 *
	 */
	public GpsViewer() {
		super("GPS Viewer");
		initialize();
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		setSize((int)WINDOW_SIZE, (int)WINDOW_SIZE + (int)TOP_INSET);
		this.setBackground(Color.BLACK);
	}
	/**
	 * draw graphics
	 */
	public void paint(Graphics g) {
		super.paint(g);
		
		// set x, y on the origin 
		Insets is = getInsets();
		int x = (getWidth() - is.left - is.right ) / 2 + is.left;
		int y = (getHeight() - (int)TOP_INSET) / 2 + (int)TOP_INSET;
		// draw background image.
		if(backgroundImgFlag){
			img = new ImageIcon(backgroundImg).getImage();
			g.drawImage(img, is.left,
						(int)TOP_INSET, getWidth() - is.left * 2,
						getHeight() - is.top - is.bottom,
						this);	// display background image
		}
		
		// draw circle
		g.setColor(Color.green);			// set circle collor
		g.drawArc(x - (int)WINDOW_SIZE / 4, y - ((int)WINDOW_SIZE - (int)TOP_INSET) / 4 - is.top, (int)WINDOW_SIZE / 2, (int)WINDOW_SIZE / 2, 0, 360);
		g.drawArc(is.left, is.top + (int)TOP_INSET, (int)WINDOW_SIZE - is.left - is.right, (int)WINDOW_SIZE - is.top - is.bottom, 0, 360);
		
		// draw lline
		g.drawLine(0, y, getWidth(), y);				// draw x line
		g.drawLine(x, (int)TOP_INSET, x, getHeight());	// draw y line
		
		// draw scale of circle
		g.setColor(Color.white);
		Font fnt = new Font("scale", Font.BOLD, 15);
		g.setFont(fnt);
		FontMetrics fm = getFontMetrics(g.getFont());
		g.drawString(String.valueOf((int)(radiusSize / 2)), x + getWidth() / 4 - is.left - fm.stringWidth(String.valueOf((int)(radiusSize / 2))), y - is.bottom);
		g.drawString(String.valueOf((int)radiusSize), getWidth() - is.left - fm.stringWidth(String.valueOf((int)radiusSize)), y - is.bottom);
		
		// draw direction
		g.drawString("North", x - is.left - fm.stringWidth("North") / 2, is.top + fm.getHeight() + (int)TOP_INSET);
		g.drawString("East", getWidth() - is.left - is.right - fm.stringWidth("East"), y + fm.getHeight());
		
		// The rate of conversion for changing into the coordinates on application from real data.
		double convertX = ((int)WINDOW_SIZE - is.left - is.right) / radiusSize / 2;		// rate of transfor to pxcel from real value.
		double convertY = ((int)WINDOW_SIZE - is.top - is.bottom) / radiusSize / 2;		// rate of transfor to pxcel from real value.
		
		// draw discription of GpsViewer
		if (targetDescription) {
			int startDescription = 35;
			int startString = 10;
			int delta = 15;
			int dSize = 10;
			int num = 0;
			
			// description of my terminal
			g.setColor(Color.YELLOW);
			g.fillArc(is.left, startDescription + delta * num, dSize, dSize, 0, 360);
			g.drawString(" : Your", is.left + dSize, startDescription + startString + delta * num);
			num++;
			
			// description of vehicle.
			g.setColor(Color.MAGENTA);
			g.fillArc(is.left, startDescription + delta * num, dSize, dSize, 0, 360);
			g.drawString(" : Vechicle", is.left + dSize, startDescription + startString + delta * num);
			num++;
			// description of pedestrian.
			g.setColor(Color.CYAN);
			g.fillArc(is.left, startDescription + delta * num, dSize, dSize, 0, 360);
			g.drawString(" : Pedestrian", is.left + dSize, startDescription + startString + delta * num);
			num++;
			// description of building.
			g.setColor(Color.ORANGE);
			g.fillRect(is.left, startDescription + delta * num, dSize, dSize);
			g.drawString(" : Building", is.left + dSize, startDescription + startString + delta * num);
			num++;
		}
		
		// display my headding.
		if (directionFlag) {
			try {
				// get predicited position lists.
				LinkedList predictedPositionLists = 
						SystemProperty.getInstance().predictPositin(InfoLogger.getInstance().getMyInfo());
				
				// get prediciotn position after 5 second.
				PredictPosition predictedPosition = (PredictPosition)predictedPositionLists.getLast();
				
				// A direction is not displayed when the case where it is not able to predict, 
				// and the terminal are not moving.
				if (predictedPosition != null) {
					// lin color change.
					String speed = 
							((HirommyPacket)InfoLogger.getInstance().getMyInfo().getLast()).getOptionInfo().getSpeed();
					if (speed == null) {
						speed = "0";
					}
					if (Double.parseDouble(speed) < 20) {
						// A color is white if speed is less than 20km.
						g.setColor(predictedColor1);
					} else if (Double.parseDouble(speed) >= 20 
								&& Double.parseDouble(speed) < 40) {
						// A color is pink if speed is 20km or more and less than 40km.
						g.setColor(Color.YELLOW);
					} else {
						// A color is red if speed is 40km or more.
						g.setColor(Color.RED);
					}
					
					HirommyPacket myInfo = (HirommyPacket)InfoLogger.getInstance().getMyInfo().getLast();
				 	// x position of prediction.
				 	int predictionX = x - targetSize / 2
	 						- (int)(convertX * DistanceCalculater.getHorizontalCordinate(myInfo.getGpsInfo().getLatitude(),
	 																					myInfo.getGpsInfo().getLongitude(),
																						String.valueOf(predictedPosition.getLatitude()),
																						String.valueOf(predictedPosition.getLongitude())))
								* MAGNI;
				 	
				 	// y position of prediction.
				 	int predictionY = y - targetSize / 2
				  			+ (int)(convertY * DistanceCalculater.getVerticalCordinate(myInfo.getGpsInfo().getLatitude(),
				  																	   myInfo.getGpsInfo().getLongitude(),
				  																	   String.valueOf(predictedPosition.getLatitude()),
				  																	   String.valueOf(predictedPosition.getLongitude())))
								* MAGNI;
				 	
					// draw direction.
				 	Line2D.Float line = new Line2D.Float(x, y, predictionX, predictionY);
				 	((Graphics2D)g).setStroke(new BasicStroke(2));		// change bold.
				 	((Graphics2D)g).draw(line);
				 	((Graphics2D)g).setStroke(new BasicStroke(1));		// change bold.
				}
				
			} catch (NullPointerException e1) {
				
			} catch (NoSuchElementException e1) {
				e1.printStackTrace();
			} catch (NumberFormatException e1) {
				e1.printStackTrace();
			}
		}
		
		// display my terminal.
		g.setColor(Color.YELLOW);
		g.fillArc(x - targetSize / 2, y - targetSize / 2, targetSize, targetSize, 0, 360);
		
		// display reaction radius.
		if (reactRadius) {
			double reactionRadius = SystemProperty.getInstance().getReactionRadius();
			g.drawArc((int)(x - reactionRadius * 2 * convertX / 2), (int)(y - reactionRadius * 2 * convertX / 2),
					   (int)(reactionRadius * 2 * convertX), (int)(reactionRadius * 2 * convertY),
					   0, 360);
		}
		
		// display other terminal.
		try {
			InfoLogger infoLogger = InfoLogger.getInstance();						// InfoLogger instance.
			LinkedList terminalLists = infoLogger.getTargetTerminal();				// get terminal list.
			HirommyPacket myInfo = (HirommyPacket)infoLogger.getMyInfo().getLast();	// My latest Infomation.
			HirommyPacket thp;														// target latest infomation.
			GpsInfo tg;																// target latest GPS infomation.
			Iterator it = terminalLists.iterator();
			
			while (it.hasNext()) {
				TargetTerminal t = (TargetTerminal)it.next();
				 thp = (HirommyPacket)t.getInfo().getLast();
				 tg = thp.getGpsInfo();
				 // if no incoming GPS infomation more than TERMINAL_LIVE, don't display the terminal.
				 if (Math.abs(Double.parseDouble(myInfo.getGpsInfo().getTime()) - Double.parseDouble(tg.getTime())) <= TERMINAL_LIVE) {
				 	// color change
				 	if (thp.getMyProfile().getTerminalType() == MyProfile.VEHICLE) {
				 		// if terminal is vehchle, color is changed.
				 		g.setColor(Color.MAGENTA);
				 	} else {
				 		// if terminal is pedestrian, color is changed.
				 		g.setColor(Color.CYAN);
				 	}
				 	// display terminal with drawing circle.
				 	// x position of terget.
				 	int targetX = x - targetSize / 2
	 						- (int)(convertX * DistanceCalculater.getHorizontalCordinate(myInfo.getGpsInfo().getLatitude(),
	 																					myInfo.getGpsInfo().getLongitude(),
																						tg.getLatitude(),
																						tg.getLongitude()));
				 	// y position of terget.
				 	int targetY = y - targetSize / 2
				  			+ (int)(convertY * DistanceCalculater.getVerticalCordinate(myInfo.getGpsInfo().getLatitude(),
				  																	  myInfo.getGpsInfo().getLongitude(),
																					  tg.getLatitude(),
																					  tg.getLongitude()));
				 	g.fillArc(targetX, targetY, targetSize, targetSize, 0, 360);
				 	
				 	// display string of target infomation.
				 	if (targetDescription) {
				 		String targetUserName = thp.getMyProfile().getUserName();
					 	g.setColor(Color.WHITE);
					 	g.drawLine(targetX + targetSize / 2, targetY, targetX + targetSize / 2 + 10, targetY - 10);
					 	g.drawString(targetUserName, targetX + targetSize / 2 + 10, targetY - 11);
				 	} // end of display string of target infomation.
				 	
				 	// display prediction position of target teraminal.
				 	if (predictionFlag) {
				 		// get prediction lists.
				 		LinkedList predictionLists = SystemProperty.getInstance().predictPositin(t.getInfo());
				 		
				 		if (predictionLists != null) {
					 		// lin color change.
							String speed = thp.getOptionInfo().getSpeed();
							if (speed == null) {
								speed = "0";
							}
							if (Double.parseDouble(speed) < 20) {
								// A color is white if speed is less than 20km.
								g.setColor(predictedColor1);
							} else if (Double.parseDouble(speed) >= 20 
										&& Double.parseDouble(speed) < 40) {
								// A color is pink if speed is 20km or more and less than 40km.
								g.setColor(Color.PINK);
							} else {
								// A color is red if speed is 40km or more.
								g.setColor(Color.RED);
							}
							
					 		Iterator predictionIterator = predictionLists.iterator();
					 		while (predictionIterator.hasNext()) {
					 			PredictPosition predictedPosition = (PredictPosition)predictionIterator.next();
					 			
							 	// x position of prediction.
							 	int predictionX = x - targetSize / 2
				 						- (int)(convertX * DistanceCalculater.getHorizontalCordinate(myInfo.getGpsInfo().getLatitude(),
				 																					 myInfo.getGpsInfo().getLongitude(),
																									 String.valueOf(predictedPosition.getLatitude()),
																									 String.valueOf(predictedPosition.getLongitude())));
							 	
							 	// y position of prediction.
							 	int predictionY = y - targetSize / 2
							  			+ (int)(convertY * DistanceCalculater.getVerticalCordinate(myInfo.getGpsInfo().getLatitude(),
							  																	   myInfo.getGpsInfo().getLongitude(),
							  																	   String.valueOf(predictedPosition.getLatitude()),
							  																	   String.valueOf(predictedPosition.getLongitude())));
							 	// draw predicted position.
							 	g.fillArc(predictionX, predictionY, targetSize - 1, targetSize - 1, 0, 360);
							 	
							 	// reaction prosess.
							 	// TODO cording reaction prosess.
							 	if (SystemProperty.getInstance().isReaction(myInfo, thp)) {
							 		// if target terminal is in reaction radius, 
							 		// this system alarm.
							 		g.setColor(Color.YELLOW);
							 		Font fontCaution = new Font("scale", Font.BOLD, 100);
							 		g.setFont(fontCaution);
							 		fm = getFontMetrics(g.getFont());
							 		g.drawString("Cautions",
							 					 (int)((WINDOW_SIZE) / 2) - fm.stringWidth("Cautions") / 2,
							 					 (int)((WINDOW_SIZE - TOP_INSET) / 2));
							 	}
					 		}
				 		}
				 	}	// end prediction process.
				 }
			}
			
			// draw building mark.
			if (buldingFlag) {
				BuildingHandler buildingHandler = BuildingHandler.getInstance();	// get BuildingHandler instance.
				it = buildingHandler.getBuildingLists().iterator();					// get buildinglists iterator.
				
				while (it.hasNext()) {
					g.setColor(Color.ORANGE);
					Building build = (Building)it.next();
					// x position of target building.
				 	int targetBuildX = x - targetSize / 2
						- (int)(convertX * DistanceCalculater.getHorizontalCordinate(myInfo.getGpsInfo().getLatitude(),
																				 	 myInfo.getGpsInfo().getLongitude(),
																					 build.getLatitude(),
																					 build.getLongitude()));
				 	
				 	// y position of terget building.
				 	int targetBuildY = y - targetSize / 2
				  			+ (int)(convertY * DistanceCalculater.getVerticalCordinate(myInfo.getGpsInfo().getLatitude(),
				  																	   myInfo.getGpsInfo().getLongitude(),
																					   build.getLatitude(),
																					   build.getLongitude()));
				 	g.fillRect(targetBuildX, targetBuildY, targetSize, targetSize);
				 	
				 	// display string of target building infomation.
				 	if (targetDescription) {
					 	g.setColor(Color.WHITE);
					 	g.drawLine(targetBuildX + targetSize / 2, targetBuildY, targetBuildX + targetSize / 2 + 10, targetBuildY - 10);
					 	g.drawString(build.getName(), targetBuildX + targetSize / 2 + 10, targetBuildY - 11);
				 	} // end of display string of target building infomation.
				}
			}
			
		} catch (NoSuchElementException e) {
			System.out.println("Error : NoSuchElementException occure in GPS Viewer.");
			e.printStackTrace();
		} catch (NullPointerException e) {
			
		}		// end of display terminal.
	}
	/**
	 * @param radiusSize The radiusSize to set.
	 */
	public void setRadiusSize(double radiusSize) {
		this.radiusSize = radiusSize;
	}
	/**
	 * @param targetSize The targetSize to set.
	 */
	public void setTargetSize(int targetSize) {
		this.targetSize = targetSize;
	}
	/**
	 * @param targetDescription The targetDescription to set.
	 */
	public void setTargetDescription(boolean targetDescription) {
		this.targetDescription = targetDescription;
	}
	/**
	 * set background of GPS viewer
	 * 
	 * @param id a number of background image
	 */
	public void setBackgroundImg(int id) {
		if(id == NONE) {
			this.backgroundImg = null;
			this.backgroundImgFlag = false;
		} else {
			this.backgroundImg = "img/" + ThemeImpl.getThemeName(id) + "/gvback.jpg";
			this.backgroundImgFlag = true;
		}
	}
	/**
	 * @return Returns the buldingFlag.
	 */
	public boolean isBuldingFlag() {
		return this.buldingFlag;
	}
	/**
	 * @param buldingFlag The buldingFlag to set.
	 */
	public void setBuldingFlag(boolean buldingFlag) {
		this.buldingFlag = buldingFlag;
	}
	/**
	 * @param directionFlag The directionFlag to set.
	 */
	public void setDirectionFlag(boolean directionFlag) {
		this.directionFlag = directionFlag;
	}
	/**
	 * @param predictionFlag The predictionFlag to set.
	 */
	public void setPredictionFlag(boolean predictionFlag) {
		this.predictionFlag = predictionFlag;
	}
	/**
	 * @param reactRadius The reactRadius to set.
	 */
	public void setReactRadius(boolean reactRadius) {
		this.reactRadius = reactRadius;
	}
}
