/*
 * Created on 2004/08/10
 *
 */
package jnet.hirommy.gui;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.util.NoSuchElementException;

import javax.swing.ImageIcon;
import javax.swing.JInternalFrame;

import jnet.hirommy.core.SystemProperty;
import jnet.hirommy.core.NetworkConst;
import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.info.MyProfileImpl;
import jnet.hirommy.util.InfoLogger;
import jnet.hirommy.util.Theme;
import jnet.hirommy.util.ThemeImpl;
import jnet.its.gps.HeaddingCalculator;
import jnet.its.gps.SimpleHeaddingCalculationMethod;

/**
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class MyPropertyViewInternalFrame extends JInternalFrame implements Theme {
	
	public static final int HEIGHT = 260;
	public static final int WIDTH = 300;
	private static final int DELTA = 14;
	private static final int TOP_SPACE = 40;
	private static final int LEFT_SPACE = 10;
	private static final int TOP_INSET = 25;
	private static final int DIVIDE_VALE = 3;
	
	// my property
	private String ipAddr = SystemProperty.getInstance().getAddr();
	private String hostName = SystemProperty.getInstance().getName();
	private int port = NetworkConst.PORT;
	
	private Image img;
	private String backgroundImg;							// background imagge of GPS Viewer
	private boolean backgroundImgFlag = false;			// tureFuse image, falseFdon't use image
	private int theme = NONE;								// terma name.
	
	// HeaddingCalculator
	private HeaddingCalculator headdingCalculator =
			new SimpleHeaddingCalculationMethod(); 
	
	/**
	 * This is the default constructor
	 */
	public MyPropertyViewInternalFrame() {
		super("My Infomation Viewer");
		initialize();
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(HEIGHT, WIDTH);
	}
	/**
	 * paint.
	 */
	public void paint(Graphics g) {
		super.paint(g);
		
		Insets is = getInsets();	// get inset.
		
		// draw background image.
		if(backgroundImgFlag){
			img = new ImageIcon(backgroundImg).getImage();
			g.drawImage(img, is.left,
						(int)TOP_INSET, getWidth() - is.left * 2,
						getHeight() - is.top - is.bottom,
						this);	// display background image
		}
		
		// my property
		String userName = MyProfileImpl.getInstance().getUserName();
		int terminal = MyProfileImpl.getInstance().getTerminalType();
		int authorization = MyProfileImpl.getInstance().getAuthorizeType();
		String serialPort = SystemProperty.getInstance().getSerialPortNumber();
		
		int num = 0;
		String str = null;
		g.setColor(ThemeImpl.getThemeStringColor(theme));	// set color of string.
		Font fnt = new Font("scale", Font.PLAIN, 13);		// set font.
		g.setFont(fnt);
		
		// display my property.
		// user name
		g.drawString("UserName", LEFT_SPACE, TOP_SPACE + DELTA * num);
		g.drawString(": " + userName, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// terminal type
		g.drawString("Terminal", LEFT_SPACE, TOP_SPACE + DELTA * num);
		if (terminal == 0) {
			str = "Vehicle";
		} else if (terminal == 1) {
			str = "Pedestrian";
		} else {
			str = "Other";
		}
		g.drawString(": " + str, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// authorization
		g.drawString("Authorization", LEFT_SPACE, TOP_SPACE + DELTA * num);
		if (authorization == 0) {
			str = "GOVERMENT";
		} else {
			str = "NONE";
		}
		g.drawString(": " + str, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// IP address
		g.drawString("IPaddress", LEFT_SPACE, TOP_SPACE + DELTA * num);
		g.drawString(": " + ipAddr, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// Hostname
		g.drawString("HostName", LEFT_SPACE, TOP_SPACE + DELTA * num);
		g.drawString(": " + hostName, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// UDPPort
		g.drawString("UDPPort", LEFT_SPACE, TOP_SPACE + DELTA * num);
		g.drawString(": " + port, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// SerialPortNumber
		g.drawString("SerialPort", LEFT_SPACE, TOP_SPACE + DELTA * num);
		g.drawString(": " + serialPort, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// display my GPS Infomation and Activity infomation.
		HirommyPacket p = null;
		try {
			p = (HirommyPacket)InfoLogger.getInstance().getMyInfo().getLast();
		} catch (NoSuchElementException e) {
			e.printStackTrace();
		}
		
		// display string.
		// time
		g.drawString("Time", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// latitude
		g.drawString("Latitude", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// longitude
		g.drawString("Longitude", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// state
		g.drawString("State", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// satellite
		g.drawString("satellite", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// altitude
		g.drawString("Altitude", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// accel
		g.drawString("Accel", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// speed
		g.drawString("Speed", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
		
		// headding
		g.drawString("Headding", LEFT_SPACE, TOP_SPACE + DELTA * num);
		num++;
			
		try {
			// display value.
			num -= 9;
			// time
			g.drawString(": " + p.getGpsInfo().getTime(), getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// latitude
			g.drawString(": " + p.getGpsInfo().getLatitude(), getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// longitude
			g.drawString(": " + p.getGpsInfo().getLongitude(), getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// state
			g.drawString(": " + p.getGpsInfo().getState(), getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// satellite
			g.drawString(": " + p.getGpsInfo().getSatellite(), getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// altitude
			g.drawString(": " + p.getGpsInfo().getAltitude() + " m", getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// accel
			g.drawString(": " + p.getOptionInfo().getAccel() + " m/s", getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// speed
			g.drawString(": " + p.getOptionInfo().getSpeed() + " km/h", getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
			// headding
			String headding = headdingCalculator.getHeddingName(p.getOptionInfo().getHeadding());
			g.drawString(": " + headding, getWidth() / DIVIDE_VALE + LEFT_SPACE, TOP_SPACE + DELTA * num);
			num++;
			
		} catch (NoSuchElementException e) {
			e.printStackTrace();
		} catch (NullPointerException e) {
			e.printStackTrace();
		}
	}
	/**
	 * set background of MyPropertyViewInternalFrame
	 * 
	 * @param id a number of background image
	 */
	public void setBackgroundImg(int id) {
		this.theme = id;
		if(id == NONE) {
			this.backgroundImg = null;
			this.backgroundImgFlag = false;
		} else {
			this.backgroundImg = "img/" + ThemeImpl.getThemeName(id) + "/miback.jpg";
			this.backgroundImgFlag = true;
		}
	}
}
