/*
 * Created on 2004/08/07
 *
 */
package jnet.hirommy.util;

import java.util.Iterator;
import java.util.LinkedList;

import jnet.hirommy.info.HirommyPacket;

/**
 * This class for logging infomation.
 * As for this class, only one exists in this program.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class InfoLogger {
	
	private static InfoLogger infoLogger = new InfoLogger();
	
	/**
	 * The history of the my infomation.
	 */
	private LinkedList myInfo = new LinkedList();
	
	/**
	 *  The history of the information received from other terminals
	 */
	private LinkedList targetTerminal = new LinkedList();
	
	/**
	 * This is the default constructor.
	 * As for this class, only one exists in this program.
	 * So A user cannot make an instance.
	 */
	private InfoLogger(){
	}
	/**
	 * This method return instance of InfoLogger.
	 * 
	 * @return instance of InfoLogger
	 */
	public static InfoLogger getInstance(){
		return infoLogger;
	}
	/**
	 * This method for logging infomation of my Hirommy Packet.
	 * 
	 * @param hp Hirommy Packet one's creating
	 */
	public void logMyHirommyPacket(HirommyPacket hp) {
		myInfo.addLast(hp);
	}
	/**
	 * This method for logging infomation of receiving from other terminal.
	 * 
	 * @param hp received Hirommy Packet
	 */
	public void logReceivedHirommyPacket(HirommyPacket hp) {
		
		boolean flag = false;						// ture:know incoming terminal, falseFdon't know
		Iterator it = targetTerminal.iterator();	// get iterator.
		
		// It searches whether the terminal which transmitted the packet exists in a history.
		while (it.hasNext()) {
			TargetTerminal tt = (TargetTerminal)it.next();
			if(tt.getIpAddr().equals(hp.getIpAddr())) {
				// If a terminal exists, a packet will be added to a history.
				tt.addInfo(hp);
				flag = true;
				break;
			}
		}
		
		if (!flag) {
			// If a terminal does not exist, a new terminal is added to a terminal list.
			targetTerminal.add(new TargetTerminal(hp.getIpAddr(), hp));
		}
	}
	
	/**
	 * This method returns a String of my infomation.
	 * 
	 * @return string of my infomation.
	 */
	public String toStringMyInfo() {
		String str = "";
		Iterator it = myInfo.iterator();
		while (it.hasNext()) {
			str = str + it.next().toString() + "\n";
		}
		return str;
	}
	
	/**
	 * This method return a String of target infomation.
	 * 
	 * @return string of target terminal infomation.
	 */
	public String toStringTargetTerminal() {
		String str = "";
		Iterator it = targetTerminal.iterator();
		while (it.hasNext()) {
			TargetTerminal tt = (TargetTerminal)it.next();
			str = str + tt.toXMLInfoHistory();
		}
		
		return str;
	}
	/**
	 * This method clear InfoLog.
	 * you can clear my infomatin and imcoming infomation that this class save.
	 *
	 */
	public void flash() {
		myInfo.clear();
		targetTerminal.clear();
	}
	
	/**
	 * @return Returns the targetTerminal.
	 */
	public LinkedList getTargetTerminal() {
		return targetTerminal;
	}
	/**
	 * @return Returns the myInfo.
	 */
	public LinkedList getMyInfo() {
		return myInfo;
	}
}
