/*
 * Created on 2004/06/21
 */
package jnet.hirommy.util;

import javax.sound.sampled.*;
import java.io.*;

/**
 * This class for playing audio.
 * Audio file must be place in /audio folder
 * and audio format must be ".wav".
 * 
 * @author Kazuya Sakai
 * @version 1.00
 * @since jdk 1.4.2
 */
public class PlayAudio implements Runnable {
	
	private String str = "audio/start.wav";			// audio file to play.
	
	/**
	 * This is default constractor.
	 * 
	 */
	public PlayAudio() {
	}
	
	/**
	 * @param str path to audio file
	 */
	public PlayAudio(String str) {
		this.str = "audio/" + str + ".wav";
	}
	
	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		File soundFile = new File(str);
		
		try {
			
			// get AudioInputStream from file.
			AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(soundFile);
			
			// get deta format of audio data.
			AudioFormat audioFormat = audioInputStream.getFormat();
			
			// get SourceDataLine.
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, audioFormat );
			SourceDataLine sourceDataLine = (SourceDataLine)AudioSystem.getLine( info );

			// open SourceDataLine.
			sourceDataLine.open( audioFormat );
			sourceDataLine.start();

			// get buffere size from frame rate and sample rate.
			int bufferSize =(int) audioFormat.getSampleRate() * audioFormat.getFrameSize();
			byte [] buffer = new byte[ bufferSize ];
			
			// output SourceDataLine as read from AudioInputStream.
			int bytesRead = 0;
			while ( bytesRead >= 0 ) {
				bytesRead = audioInputStream.read(buffer, 0, buffer.length );
				if ( bytesRead >= 0 ) {
					sourceDataLine.write( buffer, 0, bytesRead );
				}
			}
			
			// countenuas outputting until buffere is empty.
			sourceDataLine.drain();
			// close SourceDataLine.
			sourceDataLine.close();
			
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

}
