/*
 * Created on 2004/08/07
 *
 */
package jnet.hirommy.util;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;

import jnet.hirommy.info.HirommyPacket;

/**
 * This class expresses a termial that send Hirommy Packet.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class TargetTerminal implements Serializable {
	/**
	 * terminal IP Address
	 */
	protected String ipAddr = null;
	
	/**
	 * Generation time of the packet which received at the end
	 */
	protected String time = null;
	
	/**
	 *  Hirommy Packet 1sennd befor, 2second befor, 3....
	 */
	protected LinkedList infoHistoryLists = new LinkedList();
	
	/**
	 * This is the default constructor.
	 * 
	 * @param ipAddr IPAddress
	 * @param hp Hirommy Packet.
	 */
	public TargetTerminal(String ipAddr, HirommyPacket hp) {
		this.ipAddr = ipAddr;
		this.time = hp.getGpsInfo().getTime();
		infoHistoryLists.addLast(hp);
	}
	/**
	 * This method for adding Hirommy Packet to packet history.
	 * A terminal transmits the same information twice.
	 * Therefore, it is necessary to control not to Logging the same information twice.
	 * 
	 * @param hp The Hirommy Packet added to a history
	 */
	public void addInfo(HirommyPacket hp) {
		try {
			// When it differs from the packet which received already received,
			// A Hirommy Packet is added to a history.
			if (!time.equals(hp.getGpsInfo().getTime())) {
				infoHistoryLists.addLast(hp);			// add infomation to infomation history.
				this.time = hp.getGpsInfo().getTime();	// updata lately time.
			}
		} catch (NullPointerException e) {
			infoHistoryLists.addLast(hp);
			e.printStackTrace();
		}
	}
	
	/**
	 * This method return Hirommy Packet of this object.
	 * and revove first infomatin of this object's infomation history lists.
	 * This method isn't used in experience.
	 * This method is used only in simulation.
	 * 
	 * @return Hirommy Packet of this object.
	 */
	public HirommyPacket getInfoFirst() {
		return (HirommyPacket)infoHistoryLists.removeFirst();
	}
	
	/**
	 * @return Returns the info.
	 */
	public LinkedList getInfo() {
		return infoHistoryLists;
	}
	/**
	 * @return Returns the ipAddr.
	 */
	public String getIpAddr() {
		return ipAddr;
	}
	/**
	 * @return Returns the time.
	 */
	public String getTime() {
		return time;
	}
	
	/**
	 * This method for XML format of this object's infomation history.
	 * 
	 * @return XML of this object infomation history.
	 */
	public String toXMLInfoHistory() {
		String str = "";
		Iterator it = infoHistoryLists.iterator();
		while (it.hasNext()) {
			str = str + it.next().toString() + "\n";
		}
		
		return str;
	}
}
