/*
 * Created on 2004/08/29
 *
 */
package jnet.its.gps;

import java.util.LinkedList;

import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.info.OptionInfo;
import jnet.hirommy.util.DistanceCalculater;

/**
 * This is a class which realizes a simple accel calculation method.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class SimpleAccelCalculationMethod implements AccelCalculator {

	/**
	 * This is a simple accel calculation method.
	 * In this method, accelatoion is calclated by 
	 * the present speed, the speed of 1 second ago, and the distance that moved in 1 second.
	 * 
	 * @param lists Sauce information for calculation accel.
	 * @param currentGpsInfo lately GpsInfomation.
	 * @param currentSpeed lately Speed of this terminal.
	 * @return accel accel of this terminal.
	 * @see jnet.its.gps.AccelCalculator#calculateAccel(java.util.LinkedList, jnet.hiromy.info.GpsInfo, java.lang.String)
	 */
	public String calculateAccel(LinkedList lists, GpsInfo currentGpsInfo, String currentSpeed) {
		if (lists.size() >= 3) {
			// The last element is taken out.
			// / The GPS information of 1 second ago is taken out.
			GpsInfo exGpsInfo = ((HirommyPacket)lists.getLast()).getGpsInfo();
			
			// calculate the distance of present and 1 second ago.
			int d = (int)DistanceCalculater.getDistance(currentGpsInfo.getLatitude(),
													   currentGpsInfo.getLongitude(),
													   exGpsInfo.getLatitude(),
													   exGpsInfo.getLongitude());
			
			// transfor speed from (km/s) to (m/s)
			OptionInfo optinInfo = ((HirommyPacket)lists.getLast()).getOptionInfo();
			int exSpeed = (int)(Double.parseDouble(optinInfo.getSpeed()) * 1000 / 3600);
			
			int cSpeed = (int)(Double.parseDouble(currentSpeed) * 1000 / 3600);
			
			double accel;
			
			if (d != 0) {
				// accel is calculated as follow.
				// ( V^2 - V0^2 ) / 2 / d
				// V is current velocity.
				// V0 is velocity of 1 second ago.
				// d is distance which moved in 1 second.
				accel = Math.abs((cSpeed * cSpeed - exSpeed * exSpeed)) / 2 / d;
			} else {
				// if this terminal don't move (d is 0), accel equal 0.
				accel = 0;
			}
			
			return String.valueOf(accel);
			
		} else {	// When there is no GPS information for calculating speed.
			return null;
		}
	}

}
