/*
 * Created on 2004/09/06
 *
 */
package jnet.its.ivc.reaction;

import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.util.DistanceCalculater;

/**
 * This class implements ReactionAlgorithm interface.
 * The algorithm which judges whether a target terminal is in a reaction radius simply 
 * as for this class is offered.
 * 
 * @author Kazuya Sakai
 * @version 2.0
 * @since jdk 1.4.2
 */
public class SimpleReactionMethod implements ReactionAlgorithm {
	
	/**
	 * reaction radius
	 */
	private double reactionRadius;
	
	/**
	 * This is a default constractor.
	 * 
	 * @param reactionRadius radius of reaction.
	 */
	public SimpleReactionMethod(double reactionRadius) {
		this.reactionRadius = reactionRadius;
	}
	
	/**
	 * This method investigates whether the target terminal is contained in the reaction domain.
	 * 
	 * @param myHirommyPacket my hirommy packet
	 * @param targetHirommyPacket target hirommy packet
	 * @see jnet.its.ivc.reaction.ReactionAlgorithm#isReact(jnet.hirommy.info.HirommyPacket, jnet.hirommy.info.HirommyPacket)
	 */
	public boolean isReact(HirommyPacket myHirommyPacket,
			HirommyPacket targetHirommyPacket) {
		
		GpsInfo myGpsInfo = myHirommyPacket.getGpsInfo();
		GpsInfo targetGpsInfo = targetHirommyPacket.getGpsInfo();
		
		// get distance between you and target terminals.
		double d = DistanceCalculater.getDistance(myGpsInfo.getLatitude(),
													myGpsInfo.getLongitude(),
													targetGpsInfo.getLatitude(),
													targetGpsInfo.getLongitude());
		
		if (d < reactionRadius) {
			// if distance between you and target terminal, 
			// this terminal reaction.
			return true;
		} else {
			// otherwise
			return false;
		}
	}
	
	/**
	 * This method is for setting reactin radius.
	 * 
	 * @param radius reaction radius.
	 */
	public void setReactionRadius(double radius) {
		this.reactionRadius = radius;
	}
}
