/*
 * Created on 2004/08/29
 *
 */
package jnet.its.vpec.reflect;

import jnet.hirommy.core.SystemProperty;
import jnet.hirommy.info.GpsInfo;
import jnet.hirommy.info.HirommyPacket;
import jnet.hirommy.util.DistanceCalculater;
import jnet.hirommy.util.InfoLogger;

/**
 * This class is Simple Reflect Method.
 * In this algorithm, 
 * if other terminals are in a reflect radius, 
 * terminal send him infomation.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class SimpleReflectMethod implements ReflectAlgorithm {

	/**
	 * The method which investigates whether a target terminal is in a reflect radius.
	 * 
	 * @param hp Hirommy Packet.
	 * @return true : target is in reflect radius.
	 * @see jnet.its.vpec.reflect.ReflectAlgorithm#repaintReceivingInfoViewer(jnet.hirommy.info.HirommyPacket, jnet.hirommy.info.HirommyPacket)
	 */
	public boolean isInReflectRadius(HirommyPacket hp) {
		int reflectRadius = SystemProperty.getInstance().getReflectRadius();
		GpsInfo myGpsInfo = (GpsInfo)InfoLogger.getInstance().getMyInfo().getLast();
		GpsInfo targetGpsInfo = hp.getGpsInfo();
		double distance = 
				DistanceCalculater.getDistance(myGpsInfo.getLatitude(),
											   myGpsInfo.getLongitude(),
											   targetGpsInfo.getLatitude(),
											   targetGpsInfo.getLongitude());
		if (reflectRadius >= distance) {
			// When distance with a target terminal is smaller than a reflect radius.
			return true;
		} else {
			// otherwise
			return false;
		}
	}

}
